package co.com.certicamara.servicio;

import org.apache.tomcat.util.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import co.com.certicamara.dto.CertificadoDto;
import co.com.certicamara.dto.EntidadRespuesta;
import co.com.certicamara.dto.EntidadRespuestaError;
import co.com.certicamara.dto.EntidadSolicitudDto;
import co.com.certicamara.dto.ValidezDto;
import co.com.certicamara.modelos.RespuestaModelo;
import co.com.certicamara.utils.Constantes;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import java.io.*;

import java.io.ByteArrayInputStream;
import java.math.BigInteger;
import java.security.Principal;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class TrustedServicio {

	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private SoapClienteServicio soapClienteServicio;

	public Object listarCertificados(EntidadSolicitudDto entidad) {
		try {
			
			JAXBContext context = JAXBContext.newInstance(RespuestaModelo.class);
			Unmarshaller unmarshaller = context.createUnmarshaller();

			try {
				ResponseEntity<String> credenciales = soapClienteServicio.validarAutenticacion(entidad);
				RespuestaModelo respuesta = (RespuestaModelo) unmarshaller
						.unmarshal(new StringReader(credenciales.getBody()));

				if (!respuesta.getBody().getAuthNResponse().getiDReferencia()
						.equals(Constantes.AUTENTICACION_iNVALID)) {
					ResponseEntity<String> usuarioValidado = soapClienteServicio.validarUsuario(entidad);
					respuesta = (RespuestaModelo) unmarshaller.unmarshal(new StringReader(usuarioValidado.getBody()));
					if (respuesta.getBody().getSearchResponse().getDatos()) {

						ResponseEntity<String> listacertificado = soapClienteServicio.listarCertificados(entidad);						
						respuesta = (RespuestaModelo) unmarshaller.unmarshal(new StringReader(listacertificado.getBody()));

						if (respuesta.getBody().getRespuestaLecturaModelo().getResultado()
								.equals(Constantes.SUCCESS_LECTURA)) {

							if (respuesta.getBody().getRespuestaLecturaModelo().getNumElementos() > 0
									&& respuesta.getBody().getRespuestaLecturaModelo().isFin()) {
								String[] certificados = respuesta.getBody().getRespuestaLecturaModelo().getDatos()
										.getUsuario().getCertificados().getCertificados();

								List<CertificadoDto> certificadosNuevos = new ArrayList<CertificadoDto>();

								for (String certificado : certificados) {
									byte[] certificadoBytes = Base64.decodeBase64(certificado);
									CertificateFactory certificateFactory = CertificateFactory.getInstance("X.509");
									X509Certificate x509Certificado = (X509Certificate) certificateFactory
											.generateCertificate(new ByteArrayInputStream(certificadoBytes));
									// setear las fechas de la validéz del certificado
									Date antes = x509Certificado.getNotBefore();
									Date despues = x509Certificado.getNotAfter();

									Date ahora = new Date();

									if (ahora.before(despues)) {
										BigInteger serial = x509Certificado.getSerialNumber();
										Principal sujeto = x509Certificado.getSubjectDN();
										CertificadoDto nuevoCertificado = new CertificadoDto();
										SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");
										String antesF = formatDate.format(antes);
										String despuesF = formatDate.format(despues);

										ValidezDto validity = new ValidezDto(antesF, despuesF);
										String issuer = x509Certificado.getIssuerX500Principal().getName();
										nuevoCertificado.setValidity(validity);
										nuevoCertificado.setIssuer(issuer);
										nuevoCertificado.setSerial(serial.toString());
										Map<String, String> subjectInfoMap = obetnerInformacionCertificado(
												sujeto.getName());

										for (Map.Entry<String, String> entry : subjectInfoMap.entrySet()) {
											switch (entry.getKey()) {
											case "CN":
												nuevoCertificado.setCn(entry.getValue());
												break;
											case "O":
												nuevoCertificado.setO(entry.getValue());
												break;
											case "SERIALNUMBER":
												nuevoCertificado.setSerialNumber(entry.getValue());
												break;
											case "OU":
												nuevoCertificado.setOu(entry.getValue());
												break;
											default:
												break;
											}

										}
										certificadosNuevos.add(nuevoCertificado);
									}
								}
								EntidadRespuesta entidadRespuesta = new EntidadRespuesta(200, entidad.getNuip(),
										certificadosNuevos);
								return entidadRespuesta;
							} else {
								EntidadRespuestaError error = new EntidadRespuestaError("500",
										"No se han encontrado resultados",
										"No se han encontrado resultados nuip: " + entidad.getNuip());
								return error;
							}

						}

					} else {
						EntidadRespuestaError error = new EntidadRespuestaError("403", "Usuario no registrado",
								"El usuario: " + entidad.getNuip() + ", no se encuentra registrado en el sistema.");
						return error;
					}

				} else {
					EntidadRespuestaError error = new EntidadRespuestaError("401", "Credenciales incorrectas",
							"Las credenciales ingresadas no son válidas para el usuario: " + entidad.getNuip());
					return error;
				}
			} catch (Exception e) {
				logger.error("Error con las credenciales del Trusted para nuip: " + entidad.getNuip() + " " + e);
				EntidadRespuestaError error = new EntidadRespuestaError("404", "Error consultando al servico de trustedx",
						"");
				return error;
			}

		} catch (Exception e) {
			logger.error("error consultando los certificados para el nuip: " + entidad.getNuip() + " " + e);
		}
		return null;
	}

	private Map<String, String> obetnerInformacionCertificado(String x500Name) {
		Map<String, String> map = new HashMap<>();
		String[] components = x500Name.split(",");
		for (String component : components) {
			String[] keyValue = component.trim().split("=");
			if (keyValue.length == 2) {
				map.put(keyValue[0], keyValue[1]);
			}
		}
		return map;
	}

}
